package Pooru::API::V0::Controller::Media;
use v5.40;
use Mojo::Base "Mojolicious::Controller";

use List::Util qw(uniq);


sub list ($self)
{
	my $v = $self->validation;

	my $page = $v->optional("page")->num(1, undef)->param // 1;
	return $self->render(
		json => {error => "Invalid page number."},
		status => 400,
	) if $v->has_error;

	my @tags = uniq split(" ", $v->optional("tags")->param // "");
	return $self->render(
		json => {error => "Invalid tags."},
		status => 400,
	) if $v->has_error;

	return $self->render(
		json => {error => "Too many tags."},
		status => 400,
	) if @tags > Pooru::API::V0::Model::Tags::ROWS;

	my ($media, $pager);
	if (@tags > 0) {
		my @tag_ids = map {$_->{id}} $self->tags_model
		    ->get(display => [@tags])->fetchall->@*;

		return $self->render(
			json => {error => "Some of the requested tags " .
			    "don't exist."},
			status => 400,
		) if @tag_ids != @tags;

		my $paged_media_ids = $self->media_model
		    ->with_all_tags(@tag_ids)->page($page);
		my @media_ids = map {$_->{media_id}} $paged_media_ids->
		    fetchall->@*;
		return $self->render(
			json => {error => "No matching media found."},
			status => 404,
		) if @media_ids == 0;

		$media = $self->media_model->get(@media_ids)->fetchall;
		$pager = $paged_media_ids->pager;
	} else {
		my $paged_media = $self->media_model->get->page($page);
		$media = $paged_media->fetchall;
		$pager = $paged_media->pager;
	}

	return $self->render(json => {media => $media, pager => $pager});
}

sub show ($self)
{
	my $v = $self->validation;

	my $page = $v->optional("page")->num(1, undef)->param // 1;
	return $self->render(
		json => {error => "Invalid page number."},
		status => 400,
	) if $v->has_error;

	my $media = $self->media_model->get($self->stash("media_id"))->single;
	return $self->render(
		json => {error => "Media not found"},
		status => 404,
	) if !defined($media);

	my $paged_tags = $self->tags_model
	    ->ranked_for_media($media->{id})->page($page);

	return $self->render(json => {
		$media->%*,
		tags => $paged_tags->fetchall,
		pager => $paged_tags->pager,
	});
}

sub random ($self)
{
	my $v = $self->validation;

	my $no_redirect = $v->optional("no_redirect")->param;
	return $self->render(
		json => {error => "Invalid no_redirect."},
		status => 400,
	) if $v->has_error;

	my $id = $self->media_model->random_id;

	return $self->render(
		json => {error => "Media not found."},
		status => 404,
	) if !defined($id);

	return $self->render(json => $id) if $no_redirect;

	return $self->redirect_to("show_media", media_id => $id);
}

sub similar ($self)
{
	my $media = $self->media_model->get($self->stash("media_id"))->single;
	return $self->render(
		json => {error => "Media not found"},
		status => 404,
	) if !defined($media);

	my @similar_media = $self->media_model
	    ->similar($media->{id})->fetchall->@*;
	my %media = map +(
		$_->{id} => $_
	), $self->media_model
	    ->get(map {$_->{media_id}} @similar_media)->fetchall->@*;

	foreach my $sm (@similar_media) {
		$sm = {
			score => $sm->{score},
			$media{$sm->{media_id}}->%*,
		}
	}

	return $self->render(json => {media => [@similar_media]});
}

1;
