package Pooru::API::V0::Controller::Tags;
use v5.40;
use Mojo::Base "Mojolicious::Controller";


sub list ($self)
{
	my $v = $self->validation;

	my $page = $v->optional("page")->num(1, undef)->param // 1;
	return $self->render(
		json => {error => "Invalid page number."},
		status => 400,
	) if $v->has_error;

	my $paged_tags = $self->tags_model->ranked->page($page);

	return $self->render(json => {
		tags => $paged_tags->fetchall,
		pager => $paged_tags->pager,
	});
}

sub show ($self)
{
	my $tag_id_or_display = $self->stash("tag_id_or_display");
	my $key = $tag_id_or_display =~ m/^[1-9][0-9]*$/ ? "id" : "display";

	my $tag = $self->tags_model
	    ->get($key => [$tag_id_or_display])->single;
	return $self->render(
		json => {error => "Tag not found."},
		status => 404,
	) if !defined($tag);

	my $tag_with_count = $self->tags_model
	    ->ranked_for_tags($tag->{id})->single;
	return $self->render(
		json => {error => "Coherency error for tag $tag->{id}."},
		status => 500,
	) if !defined($tag_with_count);

	return $self->render(json => $tag_with_count);
}

sub random ($self)
{
	my $v = $self->validation;

	my $no_redirect = $v->optional("no_redirect")->param;
	return $self->render(
		json => {error => "Invalid no_redirect."},
		status => 400,
	) if $v->has_error;

	my $id = $self->tags_model->random_id;

	return $self->render(
		json => {error => "Tag not found."},
		status => 404,
	) if !defined($id);

	return $self->render(json => $id) if $no_redirect;

	return $self->redirect_to(
	    $self->url_for("show_tag", tag_id_or_display => $id));
}

sub search ($self)
{
	my $v = $self->validation;

	my $q = $v->required("q")->param;
	return $self->render(
		json => {error => "Missing query."},
		status => 400,
	) if $v->has_error;

	return $self->render(
		json => {tags => []},
		status => 200
	) if $q eq "";

	$q =~ s/[%\\_]/\\$&/g;
	$q .= '%';
	my @tags = $self->tags_model->search($q)->fetchall->@*;

	return $self->render(json => {tags => [@tags]});
}

1;
