package Pooru::API::V0::Model::Media;
use v5.40;

use Pooru::API::V0::Model::PagedResults;


my $slice = {};

sub where_in ($k, @v) { "$k IN (" . join(",", split(//, "?" x @v)) . ")" }


sub new ($class, $dbh) { bless {_dbh => $dbh}, $class }

sub count ($self)
{
	return $self->{_dbh}->selectrow_arrayref(q{
		SELECT COUNT(*) FROM media
	})->[0];
}

sub random_id ($self)
{
	my $row = $self->{_dbh}->selectrow_arrayref(q{
		SELECT id FROM media ORDER BY random() LIMIT 1
	});

	return defined($row) ? $row->[0] : undef;
}

sub get ($self, @ids)
{
	my $where_clause = "";
	$where_clause = "WHERE " . where_in("id", @ids) if @ids > 0;

	my $sth = $self->{_dbh}->prepare(qq{
		SELECT * FROM media $where_clause ORDER BY id DESC
	});

	return Pooru::API::V0::Model::PagedResults->new($sth, $self->ROWS,
	    $slice, @ids);
}

sub with_all_tags ($self, @tag_ids)
{
	my $where_clause = where_in("tag_id", @tag_ids);

	# "0 + ?" forces a numeric cast. Otherwise, it's interpreted as TEXT.
	my $stmt = qq{
	SELECT
		media_id
	FROM media_tag
	WHERE $where_clause
	GROUP BY media_id
	HAVING COUNT(media_id) = 0 + ?
	ORDER BY media_id DESC
	};
	my $sth = $self->{_dbh}->prepare($stmt);

	return Pooru::API::V0::Model::PagedResults->new($sth, $self->ROWS,
	    $slice, (@tag_ids, scalar @tag_ids));
}

sub similar ($self, $id)
{
	my $sth = $self->{_dbh}->prepare(qq{
	SELECT
		media_id,
		COUNT(tag_id) AS score
	FROM media_tag
	WHERE tag_id IN (
		SELECT
			tag_id
		FROM media_tag
		WHERE media_id = ?
	) AND media_id != ?
	GROUP BY media_id
	ORDER BY score DESC, media_id DESC
	LIMIT ?
	});

	return Pooru::API::V0::Model::Results->new($sth, $slice,
	    ($id, $id, $self->SIMILAR_ROWS));
}

1;
