package Pooru::API::V0::Model::PagedResults;
use v5.40;
use parent "Pooru::API::V0::Model::Results";


sub new ($class, $sth, $pgsz, $slice, @bind_values)
{
	my $self = $class->SUPER::new($sth, $slice, (@bind_values, $pgsz, 0));

	$self->{_current} = 1;
	$self->{_pgsz} = $pgsz;
	$self->{_offset} = @bind_values + 1;

	my $dbh = $sth->{Database};
	$self->{_rows} = $dbh->selectrow_arrayref(qq{
		SELECT COUNT(*) FROM ($sth->{Statement})
	}, undef, @bind_values)->[0];
	$self->{_sth} = $dbh->prepare(qq{$sth->{Statement} LIMIT ? OFFSET ?});

	{
		use integer;
		$self->{_npages} = $self->{_rows} / $self->{_pgsz} + 1;
	}

	return $self;
}

sub page_size ($self) { $self->{_pgsz} }
sub rows ($self) { $self->{_rows} }

sub page ($self, $n)
{
	$n = $n < 1 ? 1 : $n > $self->{_npages} ? $self->{_npages} : $n;
	$self->{_current} = $n;

	return $self;
}

sub pager ($self)
{
	return {
		first_page => 1,
		previous_page => $self->{_current} > 1 ?
		    $self->{_current} - 1 : undef,
		current_page => $self->{_current},
		next_page => $self->{_current} < $self->{_npages} ?
		    $self->{_current} + 1 : undef,
		last_page => $self->{_npages},
	};
}

sub fetchall ($self, $slice = $self->{_slice}) {
	$self->{_bind_values}->[$self->{_offset}] =
	    $self->{_pgsz} * ($self->{_current} - 1);

	return $self->SUPER::fetchall($slice);
}

1;
