package Pooru::API::V0::Model::Tags;
use v5.40;

use Carp;

use Pooru::API::V0::Model::PagedResults;


my $slice = {};

sub where_in ($k, @v) { "$k IN (" . join(",", split(//, "?" x @v)) . ")" }


sub new ($class, $dbh) { bless {_dbh => $dbh}, $class }

sub count ($self)
{
	return $self->{_dbh}->selectrow_arrayref(q{
		SELECT COUNT(*) FROM tag
	})->[0];
}

sub random_id ($self)
{
	my $row = $self->{_dbh}->selectrow_arrayref(q{
		SELECT display FROM tag ORDER BY random() LIMIT 1
	});

	return defined($row) ? $row->[0] : undef;
}

sub get ($self, %search)
{
	croak "Exactly one of 'id' or 'display' must be defined" if
	    !(defined($search{id}) ^^ defined($search{display}));

	my $key = defined($search{id}) ? "id" : "display";
	my $where_clause = where_in($key, $search{$key}->@*);

	my $sth = $self->{_dbh}->prepare(qq{
		SELECT * FROM tag WHERE $where_clause LIMIT ?
	});

	return Pooru::API::V0::Model::Results->new($sth, $slice,
	    ($search{$key}->@*, $self->ROWS));
}

sub _ranked ($self, $stmt, @bind_values)
{
	my $sth = $self->{_dbh}->prepare($stmt);

	return Pooru::API::V0::Model::PagedResults->new($sth, $self->ROWS,
	    $slice, @bind_values);
}

sub ranked ($self) {
	my $stmt = q{
	SELECT
		tag.id,
		tag.name,
		tag.kind_id,
		tag.display,
		COUNT(*) AS count
	FROM media_tag, tag
	WHERE media_tag.tag_id = tag.id
	GROUP BY tag.id
	ORDER BY count DESC, kind_id DESC, id ASC
	};

	return $self->_ranked($stmt);
}

sub ranked_for_tags ($self, @ids)
{
	my $where_clause = where_in("tag.id", @ids);
	my $stmt = qq{
	SELECT
		tag.id,
		tag.name,
		tag.kind_id,
		tag.display,
		COUNT(*) AS count
	FROM media_tag, tag
	WHERE media_tag.tag_id = tag.id AND $where_clause
	GROUP BY tag.id
	ORDER BY count DESC, kind_id DESC, id ASC
	};

	return $self->_ranked($stmt, @ids);
}

sub ranked_for_media ($self, @ids)
{
	my $where_clause = where_in("media_tag.media_id", @ids);
	my $stmt = qq{
	SELECT
		tag_count.id,
		tag_count.name,
		tag_count.kind_id,
		tag_count.display,
		tag_count.count
	FROM
		media_tag,
		(SELECT
			tag.id,
			tag.name,
			tag.kind_id,
			tag.display,
			COUNT(*) AS count
		FROM media_tag, tag
		WHERE media_tag.tag_id = tag.id
		GROUP BY tag.id) tag_count
	WHERE media_tag.tag_id = tag_count.id AND $where_clause
	ORDER BY count DESC, kind_id DESC, id ASC
	};

	return $self->_ranked($stmt, @ids);
}

sub search ($self, $q)
{
	my $sth = $self->{_dbh}->prepare(qq{
	SELECT
		tag.id,
		tag.name,
		tag.kind_id,
		tag.display,
		COUNT(*) AS count
	FROM media_tag, tag
	WHERE media_tag.tag_id = tag.id AND tag.name LIKE ? ESCAPE '\\'
	GROUP BY tag.id
	ORDER BY count DESC, tag.kind_id DESC, tag.id ASC
	LIMIT ?
	});

	return Pooru::API::V0::Model::Results->new($sth, $slice,
	    ($q, $self->SEARCH_ROWS));
}

1;
