package Pooru::Site::Controller::Media;
use v5.40;
use Mojo::Base "Mojolicious::Controller";


sub list ($self)
{
	my $v = $self->validation;

	my $page = $v->optional("page")->num(1, undef)->param // 1;
	return $self->render(status => 400) if $v->has_error;

	my $tags = $v->optional("tags")->param;
	return $self->render(status => 400) if $v->has_error;

	$self->title($tags) if defined($tags) && $tags ne "";

	$self->render_later;

	my $endpoint = $self->api_v0_url->path("media")
	    ->query(page => $page, tags => $tags);
	$self->ua->get_p($endpoint)->then(sub ($tx) {
		return $self->render(
			text => "Backend error.",
			status => 500,
		) if $tx->error;

		my $json = $tx->res->json;
		my @media = map +{
			show_media_url => $self->url_for("show_media",
			    media_id => $_->{id})->query(tags => $tags),
			media_src => $self->url_for_file(
			    $self->storage->get($_->{id})),
		}, $json->{media}->@*;

		$self->stash(pager => $json->{pager});
		return $self->render(
			template => "gallery",
			media => [@media],
		);
	})->wait;
}

sub show ($self)
{
	my $v = $self->validation;

	my $page = $v->optional("page")->num(1, undef)->param // 1;
	return $self->render(status => 400) if $v->has_error;

	my $tags = $v->optional("tags")->param;
	return $self->render(status => 400) if $v->has_error;

	my $media_id = $self->stash("media_id");

	$self->render_later;

	my $media_endpoint = $self->api_v0_url->path("media/$media_id")
	    ->query(page => $page);
	my $similar_endpoint = $self->api_v0_url
	    ->path("media/similar/$media_id");
	my $media_p = $self->ua->get_p($media_endpoint);
	my $similar_p = $self->ua->get_p($similar_endpoint);
	Mojo::Promise->all($media_p, $similar_p)->then(sub ($media, $similar) {
		return $self->render(
			text => "Backend error.",
			status => 500,
		) if $media->[0]->error;

		my $json = $media->[0]->res->json;
		my @tags = map +{
			name => $_->{name},
			kind_id => $_->{kind_id},
			display => $_->{display},
			count => $_->{count},
		}, $json->{tags}->@*;

		if (!$similar->[0]->error) {
			my @similar_media = map +{
				score => $_->{score},
				show_media_url => $self->url_for("show_media",
				    media_id => $_->{id})
				    ->query(tags => $tags),
				media_src => $self->url_for_file(
				    $self->storage->get($_->{id})),
			}, $similar->[0]->res->json->{media}->@*;
			$self->stash(similar_media => [@similar_media]);
		}

		$self->stash(pager => $json->{pager});
		return $self->render(
			template => "media",
			media => {
				id => $json->{id},
				filename => $json->{filename},
				content_type => $json->{content_type},
				upload_datetime => $json->{upload_datetime},
				src => $self->url_for_file(
				    $self->storage->get($json->{id})),
			},
			tags => [@tags],
		);
	})->wait;
}

sub random ($self)
{
	$self->render_later;

	my $endpoint = $self->api_v0_url
	    ->path("random/media")->query(no_redirect => 1);
	$self->ua->get_p($endpoint)->then(sub ($tx) {
		return $self->render(
			text => "Backend error.",
			status => 500,
		) if $tx->error;

		return $self->redirect_to("show_media",
		    media_id => $tx->res->json);
	})->wait;
}

1;
