package Pooru::Site::Controller::Tags;
use v5.40;
use Mojo::Base "Mojolicious::Controller";


sub list ($self)
{
	my $v = $self->validation;

	my $page = $v->optional("page")->num(1, undef)->param // 1;
	return $self->render(status => 400) if $v->has_error;

	$self->render_later;

	my $endpoint = $self->api_v0_url->path("tags")->query(page => $page);
	$self->ua->get_p($endpoint)->then(sub ($tx) {
		return $self->render(
			text => "Backend error.",
			status => 500,
		) if $tx->error;

		my $json = $tx->res->json;
		my @tags = map +{
			name => $_->{name},
			kind_id => $_->{kind_id},
			display => $_->{display},
			count => $_->{count},
		}, $json->{tags}->@*;

		$self->stash(pager => $json->{pager});
		return $self->render(
			template => "tags",
			tags => [@tags],
		);
	})->wait;
}

sub random ($self)
{
	$self->render_later;

	my $endpoint = $self->api_v0_url
	    ->path("random/tag")->query(no_redirect => 1);
	$self->ua->get_p($endpoint)->then(sub ($tx) {
		return $self->render(
			text => "Backend error.",
			status => 500,
		) if $tx->error;

		my $url = $self->url_for("list_media")
		    ->query(tags => $tx->res->json . " ");
		return $self->redirect_to($url);
	})->wait;
}

sub search ($self)
{
	my $v = $self->validation;

	my $q = $v->required("q")->param;
	return $self->render(json => [], status => 400) if $v->has_error;

	return $self->render(json => []) if $q eq "";

	$self->render_later;

	my $endpoint = $self->api_v0_url->path("search")->query(q => $q);
	$self->ua->get_p($endpoint)->then(sub ($tx) {
		return $self->render(
			json => [],
			status => 500,
		) if $tx->error;

		my $json = $tx->res->json;
		my @tags = map +{
			display => $_->{display},
			count => $_->{count},
			kind_class => $_->{kind_id} ?
			    "tag-kind-$_->{kind_id}" : "accent",
		}, $json->{tags}->@*;

		return $self->render(json => [@tags]);
	})->wait;
}

1;
